
// Lubbo's Fan Control
// Many thanks to:
// Iam Liuzhong
// Lobotomo Software
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
//


#include "stdlib.h"
#include "stdio.h"
#include "windows.h"
//Console inp and out functions
#include "conio.h"
#include "math.h"

// the temperature threshold settings used to compute the desired/target fan RPMs
float lowerTempThreshold;
float upperTempThreshold;

// the "base"/slowest/lower-limit fan RPMs
int leftFanBaseRPM;
int rightFanBaseRPM;

// the computed desired/target fan RPMs based upon the pref settings
int leftFanTargetRPM;
int rightFanTargetRPM;

int sensorControlMode;

int debug = 0;
int info = 0;

// Value to turn off(0)/on(1) debug logging
#define MFDebug       0
#define MFDebugLeft   0
#define MFDebugRight  0
#define MFDebugPrefs  0

// Value that determines the interval between Fan Control updates
// - use a shorter interval when using faster-responding sensors
// - too long an interval will lead to a oscillating fans
#define MFUpdateInterval  3500
#define MFSetKeyInteval	  1000
#define MFSetKeyIntevalShort 1000
#define MFRetryReadSMC 3 //120
#define MFRetrySetSMC 3 //120

// Values that specify the range of temperatures (in degrees Celcius) for which
// Fan Control will compute and set a fans RPM versus it's sensor's temperature
//
// NOTE: these values should correspond to the range of values available via the
//       UI's "Lower Temp Threshold" and "Upper Temp Threshold" settings
#define MFLowerTempThresholdBottom  30.0
#define MFLowerTempThresholdTop     55.0
#define MFUpperTempThresholdBottom  65.0
#define MFUpperTempThresholdTop     90.0

// Values that determine the "safe" min/max fan speeds
#define MFMinLeftFanRPM   1200
#define MFMaxLeftFanRPM   6000
//
#define MFMinRightFanRPM  1200
#define MFMaxRightFanRPM  6000

// Values that control the speed-stepping of the fans:
// - fans are adjusted in increments of MFRPMspeedStep
// - fans are adjusted to RPMs on MFRPMspeedStep boundaries
// - fans are adjusted in speed increments no larger than MFMaxRPMspeedStep
//
// NOTEs
// - the UI/slider increments should be set to ensure that the fan settings are
//   are in MFRPMspeedStep increments
#define MFRPMspeedStep     100
#define MFMaxRPMspeedStep  500

// Values that define the meaning of the radio-button selections
#define CPU_TEMP_CONTROLS_BOTH_FANS                                 0
#define CPU_TEMP_CONTROLS_LEFT_FAN_AND_GPU_TEMP_CONTROLS_RIGHT_FAN  1
#define GPU_TEMP_CONTROLS_BOTH_FANS                                 2


#define APPLESMC_DATA_PORT 0x300
/* command/status port used by Apple SMC */
#define APPLESMC_CMD_PORT 0x304

#define APPLESMC_STATUS_MASK 0x0f
#define APPLESMC_READ_CMD 0x10
#define APPLESMC_WRITE_CMD 0x11
#define APPLESMC_GET_KEY_BY_INDEX_CMD 0x12
#define APPLESMC_GET_KEY_TYPE_CMD 0x13

#define APPLESMC_MIN_WAIT 5
#define APPLESMC_MAX_WAIT 32



// key values
//
// temperature sensors
#define SMC_KEY_CPU_TEMP  "TC0D" // CPU diode temp
#define SMC_KEY_GPU_TEMP  "TG0P" // GPU processor temp
//
// current RPM for fans
#define SMC_KEY_LEFT_FAN_RPM   "F0Ac" // left fan
#define SMC_KEY_RIGHT_FAN_RPM  "F1Ac" // right fan
//
// safe minimum RPM for fans (also see MFDefinitions.h)
#define SMC_KEY_LEFT_FAN_RPM_MIN   "F0Mn" // safe minimum RPM: 1200
#define SMC_KEY_RIGHT_FAN_RPM_MIN  "F1Mn" // safe minimum RPM: 1200


SC_HANDLE hSCMan = NULL;
BOOL IsWinNT = FALSE;
BOOL IsDriverLoaded = FALSE;

// Return Value // Meaning
enum { 	
	DRIVER_ALREADY_INSTALLED=100, //100 Driver is already Installed
	DRIVER_INSTALL_SUCCESS, //101
	DRIVER_INSTALL_FAILURE, //102
	DRIVER_ALREADY_UNINSTALLED, //103
	DRIVER_UNINSTALL_SUCCESS, //104
	DRIVER_UNINSTALL_FAILURE, //105
	DRIVER_NOT_INSTALLED, //106
	DRIVER_ALREADY_STARTED, //107
	DRIVER_IN_USE
};






// SCM & GIVEIO control
BOOL InitSCM(){
	if ((hSCMan = OpenSCManager(NULL, NULL,SC_MANAGER_ALL_ACCESS)) == NULL){
		if (debug) printf("ERROR: Can't connect to Service Control Manager.\n");
		return FALSE;
	}
	return TRUE;
}

BOOL ShutDownSCM(SC_HANDLE hSCMan){
	return CloseServiceHandle(hSCMan);
}

DWORD DriverInstall(LPSTR lpPath, LPSTR lpDriver){
	BOOL dwStatus = 0;
	SC_HANDLE hService = NULL;

	// add to service control manager's database
	if (
			( hService = CreateServiceA	(	hSCMan,
					lpDriver,
					lpDriver,
					SERVICE_ALL_ACCESS,
					SERVICE_KERNEL_DRIVER,
					SERVICE_DEMAND_START,
					SERVICE_ERROR_NORMAL,
					lpPath, NULL, NULL, NULL, NULL, NULL
					)
				) == NULL
			)
	dwStatus = GetLastError();
	else
	CloseServiceHandle(hService);

	return dwStatus;
}

DWORD DriverRemove(LPSTR lpDriver){
	BOOL dwStatus = 0;
	SC_HANDLE hService = NULL;

	// get a handle to the service
	if ((hService = OpenServiceA(hSCMan, lpDriver, SERVICE_ALL_ACCESS)) != NULL)
	{
		// remove the driver
		if (!DeleteService(hService)) dwStatus = GetLastError();
	}
	else dwStatus = GetLastError();

	if (hService != NULL) CloseServiceHandle(hService);

	return dwStatus;
}

///////////////////////////////////////////////////////////////
// FUNC: GetDriverStatus
// DESC: Returns a Bool; 0 -> GiveIO Driver NOT loaded
// 1 -> GiveIO Driver LOADED
///////////////////////////////////////////////////////////////

int GetDriverStatus()
{
	return IsDriverLoaded;
}

BOOL AttachDrv()
{
	HANDLE h;

	h = CreateFileA("\\\\.\\giveio", GENERIC_READ, 0, NULL,OPEN_EXISTING,FILE_ATTRIBUTE_NORMAL, NULL);
	if(h == INVALID_HANDLE_VALUE)
	{
		if (debug) printf("ERROR: Couldn't access giveio device.\n");
		return FALSE;
	}
	CloseHandle(h);
	return TRUE;
}

BOOL AttachDIO()
{
	OSVERSIONINFO osvi;
	osvi.dwOSVersionInfoSize = sizeof(OSVERSIONINFO);
	GetVersionEx(&osvi);

	IsWinNT = osvi.dwMajorVersion == 3 || osvi.dwMajorVersion == 4 || osvi.dwMajorVersion == 5 || osvi.dwMajorVersion == 6;

	if(IsWinNT)
	{
		//Load the DirectIO Driver and attach it to this process
		printf("Is NT\n");
		//try opening SCM ; if failed Bail out
		if(!InitSCM()) return FALSE;

		//Install the Driver
		char szDrvPath[MAX_PATH];
		GetSystemDirectoryA(szDrvPath,MAX_PATH);
		lstrcatA(szDrvPath,"\\Drivers\\GiveIO.sys");
		DWORD dwRet = DriverInstall(szDrvPath,"giveio");

		if(dwRet != 0 && dwRet != 0x00000431) //Success or already installed
		{
			if (debug) printf("ERROR: Could not initialize GiveIO.sys Driver.\n");
			return FALSE;
		}

		if(AttachDrv())
		{
			IsDriverLoaded = TRUE ;
			return TRUE ; // Successful PROCESS_ATTACH
		}
		else
		{
			DriverRemove("giveio");
			return FALSE;
		}
	}

	return TRUE;
}

BOOL DetachDIO()
{
	// Perform any necessary cleanup.

	//if it is WinNT unload the giveIO driver
	if(IsWinNT)
	{
		DriverRemove("giveio");
		IsDriverLoaded = FALSE ;
		return ShutDownSCM(hSCMan); //No Error Check :-P
	}
	return FALSE;
}

short OutPort( int PortAddress, int PortData )
{
	short Dummy;

	Dummy = (short)(_outp( PortAddress, PortData ));
	return(Dummy);

}

short InPort( int PortAddress )
{
	short PortData;

	PortData = (short)(_inp( PortAddress ));
	return( PortData );

}



int wait_status(short val)
{
	unsigned int i;

	val = val & APPLESMC_STATUS_MASK;

	for (i = 0; i < 200; i++) {
		//if (debug) printf("wait %i\n",i);
		if ((InPort(APPLESMC_CMD_PORT) & APPLESMC_STATUS_MASK) == val) return 0;
		Sleep(10);
	}

	return -1;
}

/*
int wait_status(short val)
{
	unsigned int i;

	val = val & APPLESMC_STATUS_MASK;

	for (i = APPLESMC_MIN_WAIT; i < APPLESMC_MAX_WAIT; i <<= 1) {
		Sleep(i);
		if ((InPort(APPLESMC_CMD_PORT) & APPLESMC_STATUS_MASK) == val) return 0;
	}

	return -1;
}
*/

static int applesmc_read_key(const char* key, short* buffer, short len)
{
	int i;
	
	if (debug) printf("read key1\n");
	
	OutPort(APPLESMC_CMD_PORT, APPLESMC_READ_CMD);
	
	if (debug) printf("read key2\n");
	
	if (wait_status(0x0c)){
		if (debug) printf("applesmc_read_key(%s , buffer , %d) -> OutPort(%i , %i) -> wait_status(0x0c) ERROR \n",key,len,APPLESMC_CMD_PORT,APPLESMC_READ_CMD);
		return -1;
	}
	
	if (debug) printf("read key3\n");
	
	for (i = 0; i < 4; i++) {
		OutPort(APPLESMC_DATA_PORT, key[i]);
		if (wait_status(0x04)) {
			if (debug) printf("applesmc_read_key(%s , buffer , %i) -> OutPort(%s , %s) -> wait_status(0x04) ERROR \n",key,len,APPLESMC_DATA_PORT,key[i]);
			return -1;
		}
	}

	OutPort(APPLESMC_DATA_PORT, len);

	for (i = 0; i < len; i++) {
		if (wait_status(0x05)){
			if (debug) printf("applesmc_read_key(%s , %s , %i) -> OutPort(%s , %i) -> wait_status(0x05) ERROR \n",key,buffer,len,APPLESMC_DATA_PORT,i);
			return -1;
		}
		buffer[i] = InPort(APPLESMC_DATA_PORT);
	}

	return 0;
}

static int applesmc_write_key(const char* key, short* buffer, short len)
{
	int i;

	OutPort(APPLESMC_CMD_PORT, APPLESMC_WRITE_CMD);
	if (wait_status(0x0c)) return -1;

	for (i = 0; i < 4; i++) {
		OutPort(APPLESMC_DATA_PORT, key[i]);
		if (wait_status(0x04)) return -1;
	}

	OutPort(APPLESMC_DATA_PORT, len);

	for (i = 0; i < len; i++) {
		if (wait_status(0x04)) return -1;
		OutPort(APPLESMC_DATA_PORT, buffer[i]);
	}

	return 0;
}






int SMCGetFanRPM(char *key){

	int i = 0;
	short rpm[2];

	for( i = 0 ; i < MFRetryReadSMC ; i++ ){
		
		memset(rpm, 0, sizeof(rpm));

		if (applesmc_read_key(key, rpm, 2) == 0) break;
		if (i == 0) printf("Read RPM %s Error ",key);
		printf(".");
		Sleep(1000);
		
	}
	if( i == MFRetryReadSMC ){
		printf("FAILED!\n");
		return 0;
	}else{
		printf("OK!\n");
	}
	
	return (rpm[0] * 0xFF + rpm[1]) / 4;
	//return 6000;
}
double SMCGetTemperature(char *key){

	int i = 0;
	short temp[2];
	
	for( i = 0 ; i < MFRetryReadSMC ; i++ ){
		
		memset(temp, 0, sizeof(temp));

		if (applesmc_read_key(key, temp, 2) == 0) break;	
		if (i == 0) printf("Read TEMP %s Error ",key);
		printf(".");
		Sleep(1000);
	}
	
	if(i == MFRetryReadSMC){
		printf("FAILED!\n");
		return 100;
	}else{
		printf("OK!\n");
	}
	
	int intValue = (temp[0] * 256 + temp[1]) >> 2;
	
	return intValue / 64.0;
	//return (temp[0] * 0xFF + temp[1]) / 4;
}

int SMCSetFanRPM(char *key, int RPM){

	short speed[2];
	int i  = 0;
	RPM = RPM * 4;
	speed[0] = RPM >> 8;
	speed[1] = RPM & 0x00FF;

	for (i = 0; i < MFRetrySetSMC; i++)
	{
		applesmc_write_key(key, speed, 2);
		if ( i == 0 ) printf("Setting Fan RPM to %i ",RPM/4);
		printf(".");
		Sleep(MFSetKeyIntevalShort);
		applesmc_write_key(key, speed, 2);
		printf(".");
		Sleep(MFSetKeyIntevalShort);
		applesmc_write_key(key, speed, 2);
		printf(".");
		Sleep(MFSetKeyInteval);
		if (applesmc_write_key(key, speed, 2) == 0) break;
		printf(".");
	}
	if (i == MFRetrySetSMC) {
		printf("FAILED!\n");
		return 1;
	}
	printf("OK!\n");
	return 0;

}

void setFans()
{
	double CPUtemp;
	double GPUtemp;

	double leftControlTemp;
	double rightControlTemp;

	int leftFanRPM;
	int rightFanRPM;

	int adjustmentRPM;
	int alignmentRPM;

	//SMCOpen();

	leftFanRPM = SMCGetFanRPM(SMC_KEY_LEFT_FAN_RPM);
	rightFanRPM = SMCGetFanRPM(SMC_KEY_RIGHT_FAN_RPM);

	CPUtemp = SMCGetTemperature(SMC_KEY_CPU_TEMP);
	GPUtemp = SMCGetTemperature(SMC_KEY_GPU_TEMP);

	// set the "controlling sensor" temperatures that will drive the fan speeds
	leftControlTemp = CPUtemp;
	rightControlTemp = CPUtemp;
	if (sensorControlMode == GPU_TEMP_CONTROLS_BOTH_FANS) leftControlTemp = GPUtemp;
	if ((sensorControlMode == GPU_TEMP_CONTROLS_BOTH_FANS) ||
			(sensorControlMode == CPU_TEMP_CONTROLS_LEFT_FAN_AND_GPU_TEMP_CONTROLS_RIGHT_FAN))
	rightControlTemp = GPUtemp;
	if (info) printf("\n********* TEMPERATURES *********\n*\n");
	if (info) printf("*   CPU Temp = %f C \n", CPUtemp);
	if (info) printf("*   GPU Temp = %f C\n*\n", GPUtemp);
	if (info) printf("********************************\n\n");


	// ----- compute the desired/target left fan speed

	// determine the desired/target RPM indicated by the preference settings
	if (leftControlTemp < lowerTempThreshold) {
		leftFanTargetRPM = leftFanBaseRPM;
	} else if (leftControlTemp > upperTempThreshold) {
		leftFanTargetRPM = MFMaxLeftFanRPM;
	} else {
		leftFanTargetRPM = leftFanBaseRPM +
		((floor(leftControlTemp + 0.5) - lowerTempThreshold) /
		(upperTempThreshold - lowerTempThreshold) *
		(MFMaxLeftFanRPM - leftFanBaseRPM));
	}
	if (debug) printf("leftControlTemp = %f\n", leftControlTemp);
	if (debug) printf("rounded leftControlTemp = %f\n", floor(leftControlTemp + 0.5));
	if (debug) printf("ideal leftFanTargetRPM = %d\n", leftFanTargetRPM);
	if (debug) printf("leftFanRPM = %d\n", leftFanRPM);
	if (info) printf("************* FANS *************\n*\n");
	if (info) printf("*    LEFT (cpu) = %d RPM\n", leftFanRPM);
	

	// correct the fan speed if we don't have a fan-speed value/reading from smc
	if (leftFanRPM == 0) {
		leftFanRPM = leftFanTargetRPM;
		if (debug) printf("corrected leftFanRPM = %d\n", leftFanRPM);
	} /*else { // pretend fan speed is aligned to nearest MFRPMspeedStep boundary
	alignmentRPM = (leftFanRPM % MFRPMspeedStep);
	leftFanRPM = leftFanRPM - alignmentRPM;
	if (alignmentRPM > (MFRPMspeedStep / 2)) leftFanRPM = leftFanRPM + MFRPMspeedStep;
	if (MFDebugLeft) NSLog (@"aligned leftFanRPM = %d\n", leftFanRPM);
	} */

	// determine difference between fan's desired/target RPM and the current RPM
	adjustmentRPM = (leftFanTargetRPM - leftFanRPM);
	if (abs(adjustmentRPM) < (MFRPMspeedStep / 2)) {
		adjustmentRPM = 0; // current speed's within 1/2 of an RPM step, leave it
	} else { // ensure the +/- difference is not greater than the maximum allowed
		if (adjustmentRPM < -MFMaxRPMspeedStep) adjustmentRPM = -MFMaxRPMspeedStep;
		if (adjustmentRPM > MFMaxRPMspeedStep) adjustmentRPM = MFMaxRPMspeedStep;
	}
	if (debug) printf("adjustmentRPM = %d\n", adjustmentRPM);

	// compute the new desired/target RPM
	leftFanTargetRPM = leftFanRPM + adjustmentRPM;
	if (debug) printf("next leftFanTargetRPM = %d\n", leftFanTargetRPM);

	// set the desired/target RPM to the nearest MFRPMspeedStep-RPM boundary
	alignmentRPM = (leftFanTargetRPM % MFRPMspeedStep);
	leftFanTargetRPM = leftFanTargetRPM - alignmentRPM;
	if (alignmentRPM > (MFRPMspeedStep / 2)) leftFanTargetRPM = leftFanTargetRPM + MFRPMspeedStep;
	if (debug) printf("%d RPM-aligned next leftFanTargetRPM = %d\n", MFRPMspeedStep, leftFanTargetRPM);

	// when decreasing speed, don't target below the set "slowest fan speed" and
	// when increasing speeds, don't target above the maximum safe fan speed
	if (debug) printf("leftFanBaseRPM = %d\n", leftFanBaseRPM);
	if ((adjustmentRPM < 1) && (leftFanTargetRPM < leftFanBaseRPM)) leftFanTargetRPM = leftFanBaseRPM;
	if (leftFanTargetRPM > MFMaxLeftFanRPM) leftFanTargetRPM = MFMaxLeftFanRPM;
	if (debug) printf("final next leftFanTargetRPM = %d\n\n", leftFanTargetRPM);
	if (info) printf("*    -->           %d RPM\n*\n", leftFanTargetRPM);
	

	// ----- compute the desired/target right fan speed

	// determine the desired/target RPM indicated by the preference settings
	if (rightControlTemp < lowerTempThreshold) {
		rightFanTargetRPM = rightFanBaseRPM;
	} else if (rightControlTemp > upperTempThreshold) {
		rightFanTargetRPM = MFMaxRightFanRPM;
	} else {
		rightFanTargetRPM = rightFanBaseRPM +
		((floor(rightControlTemp + 0.5) - lowerTempThreshold) /
		(upperTempThreshold - lowerTempThreshold) *
		(MFMaxRightFanRPM - rightFanBaseRPM));
	}
	if (debug) printf("rightControlTemp = %f\n", rightControlTemp);
	if (debug) printf("rounded rightControlTemp = %f\n", floor(rightControlTemp + 0.5));
	if (debug) printf("ideal rightFanTargetRPM = %d\n", rightFanTargetRPM);
	if (info) printf("*    RIGHT (gpu) = %d RPM\n", rightFanRPM);
	
	// correct the fan speed if we don't have a fan-speed value/reading from smc
	if (rightFanRPM == 0) {
		rightFanRPM = rightFanTargetRPM;
		if (debug) printf("corrected rightFanRPM = %d\n", rightFanRPM);
	} /*else { // pretend fan speed is aligned to nearest MFRPMspeedStep boundary
	alignmentRPM = (rightFanRPM % MFRPMspeedStep);
	rightFanRPM = rightFanRPM - alignmentRPM;
	if (alignmentRPM > (MFRPMspeedStep / 2)) rightFanRPM = rightFanRPM + MFRPMspeedStep;
	if (MFDebugRight) NSLog (@"aligned rightFanRPM = %d\n", rightFanRPM);
	} */

	// determine difference between fan's desired/target RPM and the current RPM
	adjustmentRPM = (rightFanTargetRPM - rightFanRPM);
	if (abs(adjustmentRPM) < (MFRPMspeedStep / 2)) {
		adjustmentRPM = 0; // current speed's within 1/2 of an RPM step, leave it
	} else { // ensure the +/- difference is not greater than the maximum
		if (adjustmentRPM < -MFMaxRPMspeedStep) adjustmentRPM = -MFMaxRPMspeedStep;
		if (adjustmentRPM > MFMaxRPMspeedStep) adjustmentRPM = MFMaxRPMspeedStep;
	}
	if (debug) printf("adjustmentRPM = %d\n", adjustmentRPM);

	// compute the new desired/target RPM
	rightFanTargetRPM = rightFanRPM + adjustmentRPM;
	if (debug) printf("next rightFanTargetRPM = %d\n", rightFanTargetRPM);

	// set the desired/target RPM to the nearest MFRPMspeedStep-RPM boundary
	alignmentRPM = (rightFanTargetRPM % MFRPMspeedStep);
	rightFanTargetRPM = rightFanTargetRPM - alignmentRPM;
	if (alignmentRPM > (MFRPMspeedStep / 2)) rightFanTargetRPM = rightFanTargetRPM + MFRPMspeedStep;
	if (debug) printf("%d RPM-aligned next rightFanTargetRPM = %d\n", MFRPMspeedStep, rightFanTargetRPM);

	// when decreasing speed, don't target below the set "slowest fan speed" and
	// when increasing speeds, don't target above the maximum safe fan speed
	if (debug) printf("rightFanBaseRPM = %d\n", rightFanBaseRPM);
	if ((adjustmentRPM < 1) && (rightFanTargetRPM < rightFanBaseRPM)) rightFanTargetRPM = rightFanBaseRPM;
	if (rightFanTargetRPM > MFMaxRightFanRPM) rightFanTargetRPM = MFMaxRightFanRPM;
	if (debug) printf("final next rightFanTargetRPM = %d\n\n", rightFanTargetRPM);
	if (info) printf("*    -->           %d RPM\n*\n", leftFanTargetRPM);
	if (info) printf("********************************\n\n");
	
	// request the "target" fan speeds
	SMCSetFanRPM(SMC_KEY_LEFT_FAN_RPM_MIN, leftFanTargetRPM);
	SMCSetFanRPM(SMC_KEY_RIGHT_FAN_RPM_MIN, rightFanTargetRPM);

	//SMCClose();


}


int main(int argc, char* argv[])
{
	//short speed[2];
	int st = 0;
	int i = 0;

	// set some sane defaults
	lowerTempThreshold = MFLowerTempThresholdTop;
	upperTempThreshold = MFUpperTempThresholdTop;
	//
	leftFanBaseRPM = MFMinLeftFanRPM;
	rightFanBaseRPM = MFMinRightFanRPM;
	//
	leftFanTargetRPM = MFMinLeftFanRPM;
	rightFanTargetRPM = MFMinRightFanRPM;

	sensorControlMode = CPU_TEMP_CONTROLS_LEFT_FAN_AND_GPU_TEMP_CONTROLS_RIGHT_FAN;

	if (argc == 6){
		
		printf("Automatic Mode\n");
		AttachDIO();
		st = GetDriverStatus();
		printf("Driver Status: %d\n", st);
		if (st != 1) return 1;

		lowerTempThreshold = atoi(argv[1]);
		upperTempThreshold = atoi(argv[2]);
		leftFanBaseRPM = atoi(argv[3]);
		rightFanBaseRPM = atoi(argv[4]);
		debug = atoi(argv[5]);

		if(debug == 1){
			info = 0;
		}else if (debug == 2){
			info = 1;
			debug = 0;
		}


		while(true){

			if (debug) printf("Woked up\n");
			setFans();
			if (debug) printf("Sleeping for %i\n",MFUpdateInterval);
			Sleep(MFUpdateInterval);

		}
		
	} else if (argc == 3){
		
		printf("Manual Mode\n");
		AttachDIO();
		st = GetDriverStatus();
		printf("Driver Status: %d\n", st);
		if (st != 1) return 1;

		leftFanBaseRPM = atoi(argv[1]);
		rightFanBaseRPM = atoi(argv[2]);

		info = 1;
		
		int leftFanRPM = SMCGetFanRPM(SMC_KEY_LEFT_FAN_RPM);
		int rightFanRPM = SMCGetFanRPM(SMC_KEY_RIGHT_FAN_RPM);
		
		printf("Current Fan Speed: %i - %i\n",leftFanRPM,rightFanRPM);
		
		SMCSetFanRPM(SMC_KEY_LEFT_FAN_RPM_MIN, leftFanBaseRPM);
		SMCSetFanRPM(SMC_KEY_RIGHT_FAN_RPM_MIN, rightFanBaseRPM);
		
	} else if (argc == 2){
		printf("Info Mode\n");
		AttachDIO();
		st = GetDriverStatus();
		printf("Driver Status: %d\n", st);
		if (st != 1) return 1;

		info = 1;
		
		int leftFanRPM = SMCGetFanRPM(SMC_KEY_LEFT_FAN_RPM);
		int rightFanRPM = SMCGetFanRPM(SMC_KEY_RIGHT_FAN_RPM);
		
		printf("Current Fan Speed: %i - %i\n",leftFanRPM,rightFanRPM);
	} else {
		printf("USAGE: \n   fancontrol <MinTemp> <MaxTemp> <CPU_RPM> <GPU_RPM> <Debug=0/1/2>\nOR\n   fancontrol <CPU_RPM> <GPU_RPM>\nOR\n   fancontrol -i\n");
	}


	return 0;
}

